@@ -46,78 +46,18 @@
 #include <sha1.h>
 
 __data struct tpm_if *g_tpm = NULL;
-u16 tboot_alg_list[] = {TB_HALG_SHA1,
-                        TB_HALG_SHA256};
+u16 tboot_alg_list[] = {TB_HALG_SHA1, TB_HALG_SHA256};
+
 
-/*
- * TPM registers and data structures
- *
- * register values are offsets from each locality base
- * see {read,write}_tpm_reg() for data struct format
- */
 
-/* TPM_ACCESS_x */
-#define TPM_REG_ACCESS           0x00
-typedef union {
-    u8 _raw[1];                      /* 1-byte reg */
-    struct __packed {
-        u8 tpm_establishment   : 1;  /* RO, 0=T/OS has been established
-                                        before */
-        u8 request_use         : 1;  /* RW, 1=locality is requesting TPM use */
-        u8 pending_request     : 1;  /* RO, 1=other locality is requesting
-                                        TPM usage */
-        u8 seize               : 1;  /* WO, 1=seize locality */
-        u8 been_seized         : 1;  /* RW, 1=locality seized while active */
-        u8 active_locality     : 1;  /* RW, 1=locality is active */
-        u8 reserved            : 1;
-        u8 tpm_reg_valid_sts   : 1;  /* RO, 1=other bits are valid */
-    };
-} tpm_reg_access_t;
 
-/* TPM_STS_x */
-#define TPM_REG_STS              0x18
-typedef union {
-    u8 _raw[3];                  /* 3-byte reg */
-    struct __packed {
-        u8 reserved1       : 1;
-        u8 response_retry  : 1;  /* WO, 1=re-send response */
-        u8 self_test_done  : 1;  /* RO, only for version 2 */
-        u8 expect          : 1;  /* RO, 1=more data for command expected */
-        u8 data_avail      : 1;  /* RO, 0=no more data for response */
-        u8 tpm_go          : 1;  /* WO, 1=execute sent command */
-        u8 command_ready   : 1;  /* RW, 1=TPM ready to receive new cmd */
-        u8 sts_valid       : 1;  /* RO, 1=data_avail and expect bits are
-                                    valid */
-        u16 burst_count    : 16; /* RO, # read/writes bytes before wait */
-    };
-} tpm12_reg_sts_t;
 
-typedef union {
-    u8 _raw[4];                  /* 4-byte reg */
-    struct __packed {
-        u8 reserved1       : 1;
-        u8 response_retry  : 1;  /* WO, 1=re-send response */
-        u8 self_test_done  : 1;  /* RO, only for version 2 */
-        u8 expect          : 1;  /* RO, 1=more data for command expected */
-        u8 data_avail      : 1;  /* RO, 0=no more data for response */
-        u8 tpm_go          : 1;  /* WO, 1=execute sent command */
-        u8 command_ready   : 1;  /* RW, 1=TPM ready to receive new cmd */
-        u8 sts_valid       : 1;  /* RO, 1=data_avail and expect bits are
-                                    valid */
-        u16 burst_count    : 16; /* RO, # read/writes bytes before wait */
-        /* version >= 2 */
-        u8 command_cancel       : 1;
-        u8 reset_establishment  : 1;
-        u8 tpm_family           : 2;
-        u8 reserved2            : 4;
-    };
-} tpm20_reg_sts_t;
 
 /* Global variables for TPM status register */
 static tpm20_reg_sts_t       g_reg_sts, *g_reg_sts_20 = &g_reg_sts;
 static tpm12_reg_sts_t       *g_reg_sts_12 = (tpm12_reg_sts_t *)&g_reg_sts;
 
-static u8 g_tpm_family = 0;
+uint8_t g_tpm_family = 0;
 
 /* TPM_DATA_FIFO_x */
 #define TPM_REG_DATA_FIFO        0x24
@@ -125,6 +65,10 @@ typedef union {
         uint8_t _raw[1];                      /* 1-byte reg */
 } tpm_reg_data_fifo_t;
 
+typedef union {
+        uint8_t _raw[1];
+} tpm_reg_data_crb_t;
+
 #define TPM_ACTIVE_LOCALITY_TIME_OUT    \
           (TIMEOUT_UNIT * g_tpm->timeout.timeout_a)  /* according to spec */
 #define TPM_CMD_READY_TIME_OUT          \
@@ -159,6 +103,98 @@ static void tpm_send_cmd_ready_status(uint32_t locality)
     write_tpm_sts_reg(locality);
 }
 
+
+static bool tpm_send_cmd_ready_status_crb(uint32_t locality)
+{
+      tpm_reg_ctrl_request_t reg_ctrl_request;
+      tpm_reg_ctrl_sts_t reg_ctrl_sts;
+
+      read_tpm_reg(locality, TPM_CRB_CTRL_STS, &reg_ctrl_sts);
+
+#ifdef TPM_TRACE
+      printk(TBOOT_INFO"1. reg_ctrl_sts.tpmidle: 0x%x\n", reg_ctrl_sts.tpmidle); 	
+      printk(TBOOT_INFO"1. reg_ctrl_sts.tpmsts: 0x%x\n", reg_ctrl_sts.tpmsts); 	
+#endif
+
+	if ( reg_ctrl_sts.tpmidle== 1) {
+           reg_ctrl_request._raw[0] = 0;
+           reg_ctrl_request.cmdReady = 1;
+	    write_tpm_reg(locality, TPM_CRB_CTRL_REQ, &reg_ctrl_request);
+
+	    return true;
+	}
+
+      reg_ctrl_request._raw[0] = 0;
+      reg_ctrl_request.goIdle = 1;
+      write_tpm_reg(locality, TPM_CRB_CTRL_REQ, &reg_ctrl_request);
+	  
+      uint32_t i = 0;
+      do {
+          read_tpm_reg(locality, TPM_CRB_CTRL_REQ, &reg_ctrl_request);
+          if ( reg_ctrl_request.goIdle == 0) 
+		break;
+          else {
+              cpu_relax(); 
+	       read_tpm_reg(locality, TPM_CRB_CTRL_REQ, &reg_ctrl_request);
+
+#ifdef TPM_TRACE
+		printk(TBOOT_INFO"1. reg_ctrl_request.goIdle: 0x%x\n", reg_ctrl_request.goIdle);
+		printk(TBOOT_INFO"1. reg_ctrl_request.cmdReady: 0x%x\n", reg_ctrl_request.cmdReady);
+#endif
+
+          }
+          i++;
+       } while ( i <= TPM_DATA_AVAIL_TIME_OUT);
+
+       if ( i > TPM_DATA_AVAIL_TIME_OUT ) {
+            printk(TBOOT_ERR"TPM: reg_ctrl_request.goidle timeout!\n");
+            return false;
+       }
+
+	read_tpm_reg(locality, TPM_CRB_CTRL_STS, &reg_ctrl_sts);
+
+#ifdef TPM_TRACE
+	printk(TBOOT_INFO"2. reg_ctrl_sts.tpmidle: 0x%x\n", reg_ctrl_sts.tpmidle); 	
+       printk(TBOOT_INFO"2. reg_ctrl_sts.tpmsts: 0x%x\n", reg_ctrl_sts.tpmsts); 	
+#endif
+
+       reg_ctrl_request._raw[0] = 0;
+       reg_ctrl_request.cmdReady = 1;
+	write_tpm_reg(locality, TPM_CRB_CTRL_REQ, &reg_ctrl_request);
+
+#ifdef TPM_TRACE	
+	printk(TBOOT_INFO"2. reg_ctrl_request.goIdle: 0x%x\n", reg_ctrl_request.goIdle);
+	printk(TBOOT_INFO"2. reg_ctrl_request.cmdReady: 0x%x\n", reg_ctrl_request.cmdReady);
+#endif
+ 
+	read_tpm_reg(locality, TPM_CRB_CTRL_STS, &reg_ctrl_sts);
+
+#ifdef TPM_TRACE
+	printk(TBOOT_INFO"2. reg_ctrl_sts.tpmidle: 0x%x\n", reg_ctrl_sts.tpmidle); 	
+       printk(TBOOT_INFO"2. reg_ctrl_sts.tpmsts: 0x%x\n", reg_ctrl_sts.tpmsts); 	
+#endif
+
+	return true;
+	
+}
+
+static bool tpm_check_cmd_ready_status_crb(uint32_t locality)
+{
+    tpm_reg_ctrl_request_t reg_ctrl_request; 
+    read_tpm_reg(locality, TPM_CRB_CTRL_REQ, &reg_ctrl_request);
+
+#ifdef TPM_TRACE
+    printk(TBOOT_INFO"3. reg_ctrl_request.goIdle: 0x%x\n", reg_ctrl_request.goIdle);
+    printk(TBOOT_INFO"3. reg_ctrl_request.cmdReady: 0x%x\n", reg_ctrl_request.cmdReady);
+#endif
+
+    if ( reg_ctrl_request.cmdReady == 0) 
+		return true;
+    else
+		return false;
+
+}
+
 static bool tpm_check_cmd_ready_status(uint32_t locality)
 {
     read_tpm_sts_reg(locality);
@@ -222,7 +258,7 @@ bool tpm_validate_locality(uint32_t locality)
 {
     uint32_t i;
     tpm_reg_access_t reg_acc;
-
+    
     for ( i = TPM_VALIDATE_LOCALITY_TIME_OUT; i > 0; i-- ) {
         /*
          * TCG spec defines reg_acc.tpm_reg_valid_sts bit to indicate whether
@@ -235,13 +271,35 @@ bool tpm_validate_locality(uint32_t locality)
             return true;
         cpu_relax();
     }
-
     if ( i <= 0 )
         printk(TBOOT_ERR"TPM: tpm_validate_locality timeout\n");
 
     return false;
 }
 
+bool tpm_validate_locality_crb(uint32_t locality)
+{
+    uint32_t i;
+    tpm_reg_loc_state_t reg_loc_state;
+
+    for ( i = TPM_VALIDATE_LOCALITY_TIME_OUT; i > 0; i-- ) {
+        /*
+         *  Platfrom Tpm  Profile for TPM 2.0 SPEC
+         */
+        read_tpm_reg(locality, TPM_REG_LOC_STATE, &reg_loc_state);
+ 	 if ( reg_loc_state.tpm_reg_valid_sts == 1 && reg_loc_state.loc_assigned == 1 && reg_loc_state.active_locality == locality) {
+			 printk(TBOOT_INFO"TPM: reg_loc_state._raw[0]:  0x%x\n", reg_loc_state._raw[0]);
+			 return true;
+        	}
+        cpu_relax(); 
+    }
+
+    printk(TBOOT_ERR"TPM: tpm_validate_locality_crb timeout\n");
+    printk(TBOOT_INFO"TPM: reg_loc_state._raw[0]: 0x%x\n", reg_loc_state._raw[0]);
+    return false;
+}
+
+
 static bool tpm_wait_cmd_ready(uint32_t locality)
 {
     uint32_t            i;
@@ -276,13 +334,13 @@ static bool tpm_wait_cmd_ready(uint32_t locality)
     } while ( i <= TPM_ACTIVE_LOCALITY_TIME_OUT);
 
     if ( i > TPM_ACTIVE_LOCALITY_TIME_OUT ) {
-        printk(TBOOT_ERR"TPM: access reg request use timeout\n");
+        printk(TBOOT_ERR"TPM: FIFO_INF access reg request use timeout\n");
         return false;
     }
 
     /* ensure the TPM is ready to accept a command */
 #ifdef TPM_TRACE
-    printk(TBOOT_INFO"TPM: wait for cmd ready ");
+    printk(TBOOT_INFO"TPM: wait for cmd ready \n");
 #endif
     i = 0;
     do {
@@ -317,8 +375,43 @@ RelinquishControl:
     return false;
 }
 
-bool tpm_submit_cmd(u32 locality, u8 *in, u32 in_size,
-                    u8 *out, u32 *out_size)
+static bool tpm_wait_cmd_ready_crb(uint32_t locality)
+{
+    uint32_t i;
+
+    /* ensure the TPM is ready to accept a command */
+#ifdef TPM_TRACE
+    printk(TBOOT_INFO"TPM: wait for cmd ready \n");
+#endif
+    tpm_send_cmd_ready_status_crb(locality);
+    i = 0;
+    do {
+        if ( tpm_check_cmd_ready_status_crb(locality) )
+            break;
+        else
+            cpu_relax();
+        i++;
+    } while ( i <= TPM_CMD_READY_TIME_OUT );
+
+    if ( i > TPM_CMD_READY_TIME_OUT ) {
+        //tpm_print_status_register();
+        printk(TBOOT_INFO"TPM: tpm timeout for command_ready\n");
+        goto RelinquishControl;
+    }
+
+    return true;
+
+RelinquishControl:
+    /* deactivate current locality */
+	  //tpm_reg_loc_ctrl_t    reg_loc_ctrl;
+        //reg_loc_ctrl._raw[0] = 0;
+    //reg_loc_ctrl.relinquish = 1;
+    //write_tpm_reg(locality, TPM_REG_LOC_CTRL, &reg_loc_ctrl);
+
+    return false;
+}
+
+bool tpm_submit_cmd(u32 locality, u8 *in, u32 in_size,  u8 *out, u32 *out_size)
 {
     u32 i, rsp_size, offset;
     u16 row_size;
@@ -343,12 +436,11 @@ bool tpm_submit_cmd(u32 locality, u8 *in, u32 in_size,
         return false;
     }
 
-    if ( !tpm_wait_cmd_ready(locality) )
-        return false;
+    if ( !tpm_wait_cmd_ready(locality) )   return false;
 
 #ifdef TPM_TRACE
     {
-        printk(TBOOT_DETA"TPM: cmd size = %d\nTPM: cmd content: ", in_size);
+        printk(TBOOT_DETA"TPM: cmd size = 0x%x\nTPM: cmd content: ", in_size);
         print_hex("TPM: \t", in, in_size);
     }
 #endif
@@ -360,10 +452,8 @@ bool tpm_submit_cmd(u32 locality, u8 *in, u32 in_size,
         do {
             /* find out how many bytes the TPM can accept in a row */
             row_size = tpm_get_burst_count(locality);
-            if ( row_size > 0 )
-                break;
-            else
-                cpu_relax();
+            if ( row_size > 0 )   break;
+            else  cpu_relax();
             i++;
         } while ( i <= TPM_CMD_WRITE_TIME_OUT );
         if ( i > TPM_CMD_WRITE_TIME_OUT ) {
@@ -372,17 +462,13 @@ bool tpm_submit_cmd(u32 locality, u8 *in, u32 in_size,
             goto RelinquishControl;
         }
 
-        for ( ; row_size > 0 && offset < in_size; row_size--, offset++ )
-            write_tpm_reg(locality, TPM_REG_DATA_FIFO,
-                          (tpm_reg_data_fifo_t *)&in[offset]);
+        for ( ; row_size > 0 && offset < in_size; row_size--, offset++ )  write_tpm_reg(locality, TPM_REG_DATA_FIFO,  (tpm_reg_data_fifo_t *)&in[offset]);
     } while ( offset < in_size );
 
     i = 0;
     do {
-        if ( tpm_check_expect_status(locality) )
-            break;
-        else
-            cpu_relax();
+        if ( tpm_check_expect_status(locality) )  break;
+        else   cpu_relax();
         i++;
     } while ( i <= TPM_DATA_AVAIL_TIME_OUT );
     if ( i > TPM_DATA_AVAIL_TIME_OUT ) {
@@ -397,10 +483,8 @@ bool tpm_submit_cmd(u32 locality, u8 *in, u32 in_size,
     /* check for data available */
     i = 0;
     do {
-        if ( tpm_check_da_status(locality) )
-            break;
-        else
-            cpu_relax();
+        if ( tpm_check_da_status(locality) )  break;
+        else  cpu_relax();
         i++;
     } while ( i <= TPM_DATA_AVAIL_TIME_OUT );
     if ( i > TPM_DATA_AVAIL_TIME_OUT ) {
@@ -416,10 +500,8 @@ bool tpm_submit_cmd(u32 locality, u8 *in, u32 in_size,
         i = 0;
         do {
             row_size = tpm_get_burst_count(locality);
-            if ( row_size > 0 )
-                break;
-            else
-                cpu_relax();
+            if ( row_size > 0 )  break;
+            else cpu_relax();
             i++;
         } while ( i <= TPM_RSP_READ_TIME_OUT );
         if ( i > TPM_RSP_READ_TIME_OUT ) {
@@ -429,24 +511,19 @@ bool tpm_submit_cmd(u32 locality, u8 *in, u32 in_size,
         }
 
         for ( ; row_size > 0 && offset < *out_size; row_size--, offset++ ) {
-            if ( offset < *out_size )
-                read_tpm_reg(locality, TPM_REG_DATA_FIFO,
-                             (tpm_reg_data_fifo_t *)&out[offset]);
+            if ( offset < *out_size )  read_tpm_reg(locality, TPM_REG_DATA_FIFO, (tpm_reg_data_fifo_t *)&out[offset]);
             else {
                 /* discard the responded bytes exceeding out buf size */
                 tpm_reg_data_fifo_t discard;
-                read_tpm_reg(locality, TPM_REG_DATA_FIFO,
-                             (tpm_reg_data_fifo_t *)&discard);
+                read_tpm_reg(locality, TPM_REG_DATA_FIFO,  (tpm_reg_data_fifo_t *)&discard);
             }
 
             /* get outgoing data size */
             if ( offset == RSP_RST_OFFSET - 1 ) {
-                reverse_copy(&rsp_size, &out[RSP_SIZE_OFFSET],
-                             sizeof(rsp_size));
+                reverse_copy(&rsp_size, &out[RSP_SIZE_OFFSET], sizeof(rsp_size));
             }
         }
-    } while ( offset < RSP_RST_OFFSET ||
-              (offset < rsp_size && offset < *out_size) );
+    } while ( offset < RSP_RST_OFFSET || (offset < rsp_size && offset < *out_size) );
 
     *out_size = (*out_size > rsp_size) ? rsp_size : *out_size;
 
@@ -469,6 +546,137 @@ RelinquishControl:
     return ret;
 }
 
+
+bool tpm_submit_cmd_crb(u32 locality, u8 *in, u32 in_size,  u8 *out, u32 *out_size)
+{
+    uint32_t i;
+    
+    bool ret = true;
+
+    //tpm_reg_loc_ctrl_t reg_loc_ctrl;
+    tpm_reg_ctrl_start_t start;
+
+    tpm_reg_ctrl_cmdsize_t  CmdSize;
+    tpm_reg_ctrl_cmdaddr_t  CmdAddr;
+    tpm_reg_ctrl_rspsize_t  RspSize;
+    tpm_reg_ctrl_rspaddr_t  RspAddr;
+    uint32_t  tpm_crb_data_buffer_base;
+	
+    if ( locality >= TPM_NR_LOCALITIES ) {
+        printk(TBOOT_WARN"TPM: Invalid locality for tpm_submit_cmd_crb()\n");
+        return false;
+    }
+    if ( in == NULL || out == NULL || out_size == NULL ) {
+        printk(TBOOT_WARN"TPM: Invalid parameter for tpm_submit_cmd_crb()\n");
+        return false;
+    }
+    if ( in_size < CMD_HEAD_SIZE || *out_size < RSP_HEAD_SIZE ) {
+        printk(TBOOT_WARN"TPM: in/out buf size must be larger than 10 bytes\n");
+        return false;
+    }
+
+    if ( !tpm_validate_locality_crb(locality) ) {
+        printk(TBOOT_WARN"TPM: CRB Interface Locality %d is not open\n", locality);
+        return false;
+    }
+
+    if ( !tpm_wait_cmd_ready_crb(locality) ) {
+        printk(TBOOT_WARN"TPM: tpm_wait_cmd_read_crb failed\n");
+	 return false;
+    }
+
+#ifdef TPM_TRACE
+    {
+        printk(TBOOT_DETA"TPM: Before submit, cmd size = 0x%x\nTPM: Before submit, cmd content: ", in_size);
+        print_hex("TPM: \t", in, in_size);
+    }
+#endif
+
+    /* write the command to the TPM CRB  buffer 01-04-2016  */
+//copy *in and size to crb buffer
+
+
+
+    CmdAddr.cmdladdr = TPM_LOCALITY_CRB_BASE_N(locality) | TPM_CRB_DATA_BUFFER;
+    CmdAddr.cmdhaddr = 0;
+    RspAddr.rspaddr = TPM_LOCALITY_CRB_BASE_N(locality) | TPM_CRB_DATA_BUFFER;
+    CmdSize.cmdsize = TPMCRBBUF_LEN;
+    RspSize.rspsize = TPMCRBBUF_LEN;
+    tpm_crb_data_buffer_base = TPM_CRB_DATA_BUFFER;
+	
+
+ #ifdef TPM_TRACE  
+       printk(TBOOT_INFO"CmdAddr.cmdladdr is 0x%x\n",CmdAddr.cmdladdr);
+       printk(TBOOT_INFO"CmdAddr.cmdhaddr is 0x%x\n",CmdAddr.cmdhaddr);
+
+	printk(TBOOT_INFO"CmdSize.cmdsize is 0x%x\n",CmdSize.cmdsize);
+	printk(TBOOT_INFO"RspAddr.rspaddr is 0x%Lx\n",RspAddr.rspaddr);
+	printk(TBOOT_INFO"RspSize.rspsize is 0x%x\n",RspSize.rspsize);
+	
+#endif
+    
+    write_tpm_reg(locality, TPM_CRB_CTRL_CMD_ADDR, &CmdAddr);
+    write_tpm_reg(locality, TPM_CRB_CTRL_CMD_SIZE, &CmdSize);
+    write_tpm_reg(locality, TPM_CRB_CTRL_RSP_ADDR, &RspAddr);
+    write_tpm_reg(locality, TPM_CRB_CTRL_RSP_SIZE, &RspSize);
+    // write the command to the buffer
+    for ( i = 0 ; i< in_size; i++ )  {
+        write_tpm_reg(locality, tpm_crb_data_buffer_base++,  (tpm_reg_data_crb_t *)&in[i]);
+        //tpm_crb_data_buffer_base++;
+    }
+
+    /* command has been written to the TPM, it is time to execute it. */
+    start.start = 1;
+    write_tpm_reg(locality, TPM_CRB_CTRL_START, &start);
+    //read_tpm_reg(locality, TPM_CRB_CTRL_START, &start);
+    printk(TBOOT_INFO"tpm_ctrl_start.start is 0x%x\n",start.start);
+	
+    /* check for data available */
+    i = 0;
+    do {
+	   read_tpm_reg(locality, TPM_CRB_CTRL_START, &start);
+        //printk(TBOOT_INFO"tpm_ctrl_start.start is 0x%x\n",start.start);
+          if ( start.start == 0 ) break;
+          else  cpu_relax();
+          i++;
+    } while ( i <= TPM_DATA_AVAIL_TIME_OUT );
+
+    if ( i > TPM_DATA_AVAIL_TIME_OUT ) {
+        printk(TBOOT_ERR"TPM: wait for data available timeout\n");
+        ret = false;
+        goto RelinquishControl;
+    }
+
+    tpm_crb_data_buffer_base = TPM_CRB_DATA_BUFFER;
+
+    for ( i = 0 ; i< *out_size; i++ )  {
+        read_tpm_reg(locality, tpm_crb_data_buffer_base++, (tpm_reg_data_crb_t *)&out[i]);
+        //tpm_crb_data_buffer_base++;
+    }
+
+  
+
+#ifdef TPM_TRACE
+    {
+        printk(TBOOT_INFO"TPM: After cmd submit, response size = 0x%x\n", *out_size);
+        printk(TBOOT_DETA"TPM: After cmd submit, response content: ");
+        print_hex("TPM: \t", out, *out_size);
+    }
+#endif
+
+    //tpm_send_cmd_ready_status_crb(locality);
+
+RelinquishControl:
+    /* deactivate current locality */
+   // reg_loc_ctrl._raw[0] = 0;
+    //reg_loc_ctrl.relinquish = 1;
+    //write_tpm_reg(locality, TPM_REG_LOC_CTRL, &reg_loc_ctrl);
+
+    return ret;
+
+}
+
+
 bool release_locality(uint32_t locality)
 {
     uint32_t i;
@@ -476,13 +684,11 @@ bool release_locality(uint32_t locality)
     printk(TBOOT_DETA"TPM: releasing locality %u\n", locality);
 #endif
 
-    if ( !tpm_validate_locality(locality) )
-        return true;
+    if ( !tpm_validate_locality(locality) )   return true;
 
     tpm_reg_access_t reg_acc;
     read_tpm_reg(locality, TPM_REG_ACCESS, &reg_acc);
-    if ( reg_acc.active_locality == 0 )
-        return true;
+    if ( reg_acc.active_locality == 0 )    return true;
 
     /* make inactive by writing a 1 */
     reg_acc._raw[0] = 0;
@@ -503,41 +709,158 @@ bool release_locality(uint32_t locality)
     return false;
 }
 
+bool tpm_relinquish_locality_crb(uint32_t locality)
+{
+    uint32_t i;
+    tpm_reg_loc_state_t reg_loc_state;
+    tpm_reg_loc_ctrl_t reg_loc_ctrl;
+	
+#ifdef TPM_TRACE
+    printk(TBOOT_DETA"TPM: releasing CRB_INF locality %u\n", locality);
+#endif
+
+    if ( !tpm_validate_locality_crb(locality) )   return true;
+    read_tpm_reg(locality, TPM_REG_LOC_STATE, &reg_loc_state);
+    if ( reg_loc_state.loc_assigned == 0 )    return true;
+
+    /* make inactive by writing a 1 */
+    reg_loc_ctrl._raw[0] = 0;
+    reg_loc_ctrl.relinquish = 1;
+    write_tpm_reg(locality, TPM_REG_LOC_CTRL, &reg_loc_ctrl);
+
+    i = 0;
+    do {
+        read_tpm_reg(locality, TPM_REG_LOC_STATE, &reg_loc_state);
+        if ( reg_loc_state.loc_assigned == 0 )    return true;
+        else cpu_relax();
+        i++;
+    } while ( i <= TPM_ACTIVE_LOCALITY_TIME_OUT );
+
+    printk(TBOOT_INFO"TPM: CRB_INF release locality timeout\n");
+    return false;
+}
+
+
+
+bool is_tpm_crb(void)
+{      
+     tpm_crb_interface_id_t crb_interface;
+     read_tpm_reg(0, TPM_INTERFACE_ID, &crb_interface);
+     if (crb_interface.interface_type == TPM_INTERFACE_ID_CRB  ) {
+	 printk(TBOOT_INFO"TPM: PTP CRB interface is active...\n");
+	 if (g_tpm_family != TPM_IF_20_CRB ) g_tpm_family = TPM_IF_20_CRB;
+        return true;
+     }
+     if (crb_interface.interface_type == TPM_INTERFACE_ID_FIFO_20) {
+	  printk(TBOOT_INFO"TPM: TPM 2.0 FIFO interface is active...\n");     
+	  if (g_tpm_family != TPM_IF_20_FIFO) g_tpm_family = TPM_IF_20_FIFO;
+     }
+     return false;	
+}
+
+
 bool prepare_tpm(void)
 {
     /*
      * must ensure TPM_ACCESS_0.activeLocality bit is clear
      * (: locality is not active)
      */
-
-    return release_locality(0);
+   if (is_tpm_crb()) 
+//   	return release_locality_crb(0);
+       return true;
+   else 
+   	return release_locality(0);
 }
 
-bool tpm_detect(void)
-{
-    g_tpm = &tpm_12_if; /* Don't leave g_tpm as NULL*/
-    if ( !tpm_validate_locality(0) ) {
-        printk(TBOOT_ERR"TPM: Locality 0 is not open\n");
+bool tpm_request_locality_crb(uint32_t locality){
+
+    uint32_t            i;
+    tpm_reg_loc_state_t  reg_loc_state;
+    tpm_reg_loc_ctrl_t    reg_loc_ctrl;
+    /* request access to the TPM from locality N */
+    reg_loc_ctrl._raw[0] = 0;
+    reg_loc_ctrl.requestAccess = 1;
+    write_tpm_reg(locality, TPM_REG_LOC_CTRL, &reg_loc_ctrl);
+
+    i = 0;
+    do {
+        read_tpm_reg(locality, TPM_REG_LOC_STATE, &reg_loc_state);
+        if ( reg_loc_state.active_locality == locality && reg_loc_state.loc_assigned == 1)
+            break;
+        else
+            cpu_relax();
+        i++;
+    } while ( i <= TPM_ACTIVE_LOCALITY_TIME_OUT);
+
+    if ( i > TPM_ACTIVE_LOCALITY_TIME_OUT ) {
+        printk(TBOOT_ERR"TPM: access loc request use timeout\n");
         return false;
     }
 
-    /* determine TPM family from command check */
-    if ( g_tpm->check() )
-        g_tpm_family = 0;
-    else
-        g_tpm_family = 1;
-    printk(TBOOT_INFO"TPM: TPM Family 0x%d\n", g_tpm_family);
-    if (g_tpm_family == 1)
-        g_tpm = &tpm_20_if;
-    else
-        g_tpm = &tpm_12_if;
+    return true;
 
-    g_tpm->cur_loc = 0;
+}
+
+
+bool tpm_detect(void)
+{
+    if (is_tpm_crb()) {
+         printk(TBOOT_INFO"TPM: This is Intel PTT, TPM Family 0x%d\n", g_tpm_family);
+         if (!txt_is_launched()) {
+               if ( tpm_validate_locality_crb(0) ) 
+	             printk(TBOOT_INFO"TPM: CRB_INF Locality 0 is open\n");
+		 else {
+		 	printk(TBOOT_INFO"TPM: CRB_INF request access to Locality 0...\n");
+			if (!tpm_request_locality_crb(0)) {
+			        printk(TBOOT_ERR"TPM: CRB_INF Locality 0 request failed...\n");
+				 return false;
+			 }
+                }
+	  }
+    	  else {
+              if ( tpm_validate_locality_crb(2) ) 
+		     printk(TBOOT_INFO"TPM: CRB_INF Locality 2 is open\n");
+		else {	 
+		      printk(TBOOT_INFO"TPM: CRB_INF request access to Locality 2...\n");
+		      if (!tpm_request_locality_crb(2)) {
+		 	     printk(TBOOT_ERR"TPM: CRB_INF Locality 2 request failed...\n");
+                          return false;
+			}
+		}
+    	  }
+    }
+    else {
+		g_tpm = &tpm_12_if; /* Don't leave g_tpm as NULL*/
+		if ( tpm_validate_locality(0) )  printk(TBOOT_INFO"TPM: FIFO_INF Locality 0 is open\n");
+		else {	
+			printk(TBOOT_ERR"TPM: FIFO_INF Locality 0 is not open\n");
+			return false;
+			}
+		/* determine TPM family from command check */
+		if ( g_tpm->check() )  {
+			g_tpm_family = TPM_IF_12;
+			printk(TBOOT_INFO"TPM: discrete TPM1.2 Family 0x%d\n", g_tpm_family);	
+			}
+		else {
+			g_tpm_family = TPM_IF_20_FIFO;
+			printk(TBOOT_INFO"TPM: discrete TPM2.0 Family 0x%d\n", g_tpm_family);
+			}
+	}
+   
+    if (g_tpm_family == TPM_IF_12)  g_tpm = &tpm_12_if;
+    if (g_tpm_family == TPM_IF_20_FIFO)  g_tpm = &tpm_20_if;
+    if (g_tpm_family == TPM_IF_20_CRB)  g_tpm = &tpm_20_if;
+
+   /*  if (!txt_is_launched()) 
+	   g_tpm->cur_loc = 0;
+     else 
+	   g_tpm->cur_loc = 2;
+	 	
     g_tpm->timeout.timeout_a = TIMEOUT_A;
     g_tpm->timeout.timeout_b = TIMEOUT_B;
     g_tpm->timeout.timeout_c = TIMEOUT_C;
     g_tpm->timeout.timeout_d = TIMEOUT_D;
-
+*/
     return g_tpm->init(g_tpm);
 }
 
@@ -547,11 +870,9 @@ void tpm_print(struct tpm_if *ti)
         return;
 
     printk(TBOOT_INFO"TPM attribute:\n");
-    printk(TBOOT_INFO"\t extend policy %d\n", ti->extpol);
-    printk(TBOOT_INFO"\t current alg id 0x%x\n", ti->cur_alg);
-    printk(TBOOT_INFO"\t timeout values: A: %u, B: %u, C: %u, D: %u\n",
-            ti->timeout.timeout_a, ti->timeout.timeout_b, ti->timeout.timeout_c,
-            ti->timeout.timeout_d);
+    printk(TBOOT_INFO"\t extend policy: %d\n", ti->extpol);
+    printk(TBOOT_INFO"\t current alg id: 0x%x\n", ti->cur_alg);
+    printk(TBOOT_INFO"\t timeout values: A: %u, B: %u, C: %u, D: %u\n", ti->timeout.timeout_a, ti->timeout.timeout_b, ti->timeout.timeout_c, ti->timeout.timeout_d);
 }
 
 /*
